#clear the memory
rm(list=ls())

#seed of the random number generator
set.seed(10)

#data generator
generate.data <- function(n){
  x2 <- runif(n)
  x1 <- runif(n)
  y <- factor(ifelse((x2>2*x1)|(x2>(2-2*x1)),1,2))
  return(data.frame(x1,x2,y))
}

#generate the training set
dtrain <- generate.data(30)

#plotting
plot(dtrain$x1,dtrain$x2,col=c("blue","red")[dtrain$y],xlim=c(0,1),ylim=c(0,1))
abline(0,2)
abline(2,-2)

#generate the test set
dtest <- generate.data(5000)

#plotting
plot(dtest$x1,dtest$x2,col=c("blue","red")[dtest$y])
abline(0,2)
abline(2,-2)

#e1071
library(e1071)

#fit a linear svm
mlin <- svm(y ~ x1+x2, data=dtrain, kernel="linear",scale=F)
print(mlin)

#insert them into the graph
plot(dtrain$x1,dtrain$x2,col=c("blue","red")[dtrain$y],xlim=c(0,1),ylim=c(0,1))
points(dtrain$x1[mlin$index],dtrain$x2[mlin$index],pch=5,cex=1.75,col=rgb(0,0,0))
abline(0,2)
abline(2,-2)

#intercept
beta.0 <- -mlin$rho
print(beta.0)

#coefficients
beta.1 <- sum(mlin$coefs*dtrain$x1[mlin$index])
beta.2 <- sum(mlin$coefs*dtrain$x2[mlin$index])
print(paste(beta.1,beta.2))

#the frontier
plot(dtrain$x1,dtrain$x2,col=c("blue","red")[dtrain$y],xlim=c(0,1),ylim=c(0,1))
abline(-beta.0/beta.2,-beta.1/beta.2,col="green")

#checking
ylin.train <- predict(mlin,dtrain)
table(dtrain$y,ylin.train)

#using a polynomial kernel
#mpoly <- svm(y ~ x1+x2, data=dtrain, kernel="polynomial", scale=F, coef0 = 1, degree=2)
mpoly <- svm(y ~ x1+x2, data=dtrain, kernel="polynomial", scale=F, cost=1000, coef0=1, degree=2)
print(mpoly)

#insert them into the graph
plot(dtrain$x1,dtrain$x2,col=c("blue","red")[dtrain$y],xlim=c(0,1),ylim=c(0,1))
points(dtrain$x1[mpoly$index],dtrain$x2[mpoly$index],pch=5,cex=1.75,col=rgb(0,0,0))
abline(0,2)
abline(2,-2)

#prediction
ypoly.test <- predict(mpoly,dtest)

#confusion matrix
mc.poly <- table(dtest$y,ypoly.test)
print(mc.poly)

#error rate
err.poly <- 1-sum(diag(mc.poly))/sum(mc.poly)
print(err.poly)

#plotting
plot(dtest$x1,dtest$x2,col=c("blue","red")[ypoly.test],xlim=c(0,1),ylim=c(0,1))
abline(0,2,lwd=2)
abline(2,-2,lwd=2)

#using a rbf kernel
#mrbf <- svm(y ~ x1+x2, data=dtrain, kernel="radial", scale=F)
mrbf <- svm(y ~ x1+x2, data=dtrain, kernel="radial", scale=F,cost=1000,gamma=1)
print(mrbf)

#insert them into the graph
plot(dtrain$x1,dtrain$x2,col=c("blue","red")[dtrain$y],xlim=c(0,1),ylim=c(0,1))
points(dtrain$x1[mrbf$index],dtrain$x2[mrbf$index],pch=5,cex=1.75,col=rgb(0,0,0))

#prediction
yrbf.test <- predict(mrbf,dtest)

#confusion matrix
mc.rbf <- table(dtest$y,yrbf.test)
print(mc.rbf)

#error rate
err.rbf <- 1-sum(diag(mc.rbf))/sum(mc.rbf)
print(err.rbf)

#plotting
plot(dtest$x1,dtest$x2,col=c("blue","red")[yrbf.test],xlim=c(0,1),ylim=c(0,1))
abline(0,2,lwd=2)
abline(2,-2,lwd=2)
